
<script src="https://cdn.tailwindcss.com"></script>
<link rel="stylesheet" type="text/css" href="https://unpkg.com/@fortawesome/fontawesome-free@6.7.2/css/all.min.css"></link>
<script src="https://cdn.jsdelivr.net/npm/@json-editor/json-editor@latest/dist/jsoneditor.min.js"></script>
<script>
    JSONEditor.defaults.theme = 'tailwind';
    JSONEditor.defaults.iconlib = 'fontawesome5';
    tailwind.config = {
        theme: {
            extend: {
                colors: {
                    edgy: {
                        pink: "{{ sidebar_bg_colour }}"
                    }
                }
            }
        }
    }
</script>
<style>
    .loader {
        border-top-color: #3490dc;
        animation: spin 1s linear infinite;
    }
    @keyframes spin {
        0% { transform: rotate(0deg); }
        100% { transform: rotate(360deg); }
    }
</style>
