<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Page Not Found - 404</title>
    {% include 'admin/base_head.html.jinja' %}
</head>
<body class="bg-gray-100 min-h-screen flex items-center justify-center">
    <div class="text-center">
        <h1 class="text-9xl font-bold text-gray-700">404</h1>
        <p class="text-2xl md:text-3xl font-light leading-normal mb-8">
            Sorry, we couldn't find the page you're looking for.
        </p>
        <p class="text-gray-600 mb-8">
            The page you requested might have been moved or doesn't exist.
        </p>
        <a href="/" class="text-blue-500 hover:underline text-lg">
            Go back to the homepage
        </a>
    </div>
</body>
</html>
